@@ -21,6 +21,10 @@ extern char NoClass[];
 extern char NoResource[];
 
 void setup_focus_policy(FvwmWindow *tmp_win);
+void setup_window_font(
+  FvwmWindow *tmp_win, window_style *pstyle, Bool do_destroy);
+void setup_icon_font(
+  FvwmWindow *tmp_win, window_style *pstyle, Bool do_destroy);
 void setup_style_and_decor(
   FvwmWindow *tmp_win, window_style *pstyle, short *buttons);
 void setup_auxiliary_windows(
@@ -44,6 +48,6 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin);
 void GetWindowSizeHints(FvwmWindow *);
 void free_window_names (FvwmWindow *tmp, Bool nukename, Bool nukeicon);
 void destroy_window(FvwmWindow *);
-void RestoreWithdrawnLocation(FvwmWindow *, Bool);
+void RestoreWithdrawnLocation (FvwmWindow *tmp, Bool restart, Window parent);
 
 #endif /* ADD_WINDOW_H */
