@@ -19,7 +19,14 @@
 void DispatchEvent(Bool preserve_Tmp_win);
 int GetContext(FvwmWindow *, XEvent *, Window *dummy);
 int My_XNextEvent(Display *dpy, XEvent *event);
+int flush_expose(Window w);
+int flush_accumulate_expose(Window w, XEvent *e);
+void handle_all_expose(void);
+Bool StashEventTime(XEvent *ev);
+void CoerceEnterNotifyOnCurrentWindow(void);
 void InitEventHandlerJumpTable(void);
+
+/* event handlers */
 void HandleEvents(void);
 void HandleExpose(void);
 void HandleFocusIn(void);
@@ -42,7 +49,9 @@ void HandleClientMessage(void);
 void HandlePropertyNotify(void);
 void HandleKeyPress(void);
 void HandleVisibilityNotify(void);
-STROKE_CODE(void HandleButtonRelease(void));
-STROKE_CODE(void HandleMotionNotify(void));
+STROKE_CODE(void HandleButtonRelease(void);)
+STROKE_CODE(void HandleMotionNotify(void);)
+void WaitForButtonsUp(Bool do_handle_expose);
+int discard_events(long event_mask);
 
 #endif /* _EVENTS_ */
