@@ -53,6 +53,7 @@
 #include "libs/Colorset.h"
 #include "icccm2.h"
 #include "gnome.h"
+#include "icons.h"
 #include "add_window.h"
 #include "fvwmsignal.h"
 #include "colormaps.h"
@@ -64,6 +65,7 @@
 #include "colors.h"
 #include "focus.h"
 #include "update.h"
+#include "window_flags.h"
 
 #include <X11/Xproto.h>
 #include <X11/Xatom.h>
@@ -94,7 +96,7 @@ int master_pid;			/* process number of 1st fvwm process */
 
 ScreenInfo Scr;		        /* structures for the screen */
 MenuInfo Menus;                 /* structures for menus */
-Display *dpy;			/* which display are we talking to */
+Display *dpy = NULL;		/* which display are we talking to */
 
 Bool fFvwmInStartup = True;     /* Set to False when startup has finished */
 Bool DoingCommandLine = False;	/* Set True before each cmd line arg */
@@ -114,6 +116,7 @@ void SaveDesktopState(void);
 void SetMWM_INFO(Window window);
 void SetRCDefaults(void);
 void StartupStuff(void);
+static void catch_exit(void);
 static RETSIGTYPE SigDone(int);
 static RETSIGTYPE Restart(int);
 static RETSIGTYPE ReapChildren(int);
@@ -373,6 +376,7 @@ int main(int argc, char **argv)
     fvwm_msg(ERR,"main","can't open display %s", XDisplayName(display_name));
     exit (1);
   }
+  atexit(catch_exit);
   Scr.screen= DefaultScreen(dpy);
   Scr.NumberOfScreens = ScreenCount(dpy);
 
@@ -467,12 +471,6 @@ int main(int argc, char **argv)
     exit(1);
   }
 
-  {
-    Cursor cursor = XCreateFontCursor(dpy, XC_watch);
-    XGrabPointer(dpy, Scr.Root, 0, 0, GrabModeAsync, GrabModeAsync,
-		 None, cursor, CurrentTime);
-  }
-
   if (visualClass != -1) {
     /* grab the best visual of the required class */
     XVisualInfo template, *vizinfo;
@@ -582,6 +580,14 @@ int main(int argc, char **argv)
 
   XSetErrorHandler(FvwmErrorHandler);
 
+  {
+    /* do not grab the pointer earlier because if fvwm exits with the pointer
+     * grabbed while a different display is visible, XFree 4.0 freezes. */
+    Cursor cursor = XCreateFontCursor(dpy, XC_watch);
+    XGrabPointer(dpy, Scr.Root, 0, 0, GrabModeAsync, GrabModeAsync,
+		 None, cursor, CurrentTime);
+  }
+
   {
     Atom atype;
     int aformat;
@@ -741,10 +747,25 @@ int main(int argc, char **argv)
     DBUG("main","Unknown FVWM run-state");
   }
 
-  return 0;
+  exit(0);
 }
 
 
+/* exit handler that will try to release any grabs */
+static void catch_exit(void)
+{
+  if (dpy != NULL)
+  {
+    /* Don't care if this is called from an X error handler. We *have* to try
+     * this, whatever happens. XFree 4.0 may freeze if we don't do this. */
+    XUngrabServer(dpy);
+    XUngrabPointer(dpy, CurrentTime);
+    XUngrabKeyboard(dpy, CurrentTime);
+  }
+
+  return;
+}
+
 /*
 ** StartupStuff
 **
@@ -814,31 +835,48 @@ void CaptureOneWindow(
   FvwmWindow *fw, Window window, Window keep_on_top_win, Window parent_win)
 {
   Window w;
-  Atom atype;
-  int aformat;
-  unsigned long nitems, bytes_remain;
-  unsigned char *prop = NULL;
   unsigned long data[1];
 
+  isIconicState = DontCareState;
   if (fw == NULL)
+  {
     return;
+  }
+  if (IS_SCHEDULED_FOR_DESTROY(fw))
+  {
+    /* Fvwm might crash in complex functions if we really try to the dying
+     * window here because AddWindow() may fail and leave a destroyed window
+     * in Tmp_win.  Any, by the way, it is pretty useless to recapture a
+     * window that will vanish in a moment. */
+    return;
+  }
+  /* Grab the server to make sure the window does not die during the recapture.
+   */
+  MyXGrabServer(dpy);
+  if (!XGetGeometry(dpy, fw->w, &JunkRoot, &JunkX, &JunkY, &JunkWidth,
+		   &JunkHeight, &JunkBW,  &JunkDepth))
+  {
+    /* The window has already died, do not recapture it! */
+    MyXUngrabServer(dpy);
+    return;
+  }
   if (XFindContext(dpy, window, FvwmContext, (caddr_t *)&fw) != XCNOENT)
   {
     Bool f = PPosOverride;
+    Bool is_mapped = IS_MAPPED(fw);
 
     PPosOverride = True;
-    isIconicState = DontCareState;
-    if(XGetWindowProperty(dpy, fw->w, _XA_WM_STATE, 0L, 3L, False,
-			  _XA_WM_STATE, &atype, &aformat, &nitems,
-			  &bytes_remain, &prop) == Success)
+    if (IS_ICONIFIED(fw))
     {
-      if(prop != NULL)
-      {
-	isIconicState = *(long *)prop;
-	isIconifiedByParent =
-	  (isIconicState == IconicState) ? IS_ICONIFIED_BY_PARENT(fw) : 0;
-	XFree(prop);
-      }
+      isIconicState = IconicState;
+      isIconifiedByParent = IS_ICONIFIED_BY_PARENT(fw);
+    }
+    else
+    {
+      isIconicState = NormalState;
+      isIconifiedByParent = 0;
+      if (Scr.CurrentDesk != fw->Desk)
+	SetMapStateProp(fw, NormalState);
     }
     data[0] = (unsigned long) fw->Desk;
     XChangeProperty (dpy, fw->w, _XA_WM_DESKTOP, _XA_WM_DESKTOP, 32,
@@ -852,20 +890,23 @@ void CaptureOneWindow(
     XSelectInput(dpy, fw->w, 0);
     w = fw->w;
     XUnmapWindow(dpy, fw->frame);
-    /* This unmap may cause some applications think that they are iconified
-     * (e.g. ddd).  Thus, when the window is remapped, it will become iconic.
-     */
     RestoreWithdrawnLocation(fw, True, parent_win);
     SET_DO_REUSE_DESTROYED(fw, 1); /* RBW - 1999/03/20 */
     destroy_window(fw);
     Event.xmaprequest.window = w;
     HandleMapRequestKeepRaised(keep_on_top_win, fw);
+    if (!fFvwmInStartup)
+    {
+      SET_MAP_PENDING(fw, 0);
+      SET_MAPPED(fw, is_mapped);
+    }
     /* Clean out isIconicState here, otherwise all new windos may start
      * iconified. */
     isIconicState = DontCareState;
     /* restore previous value */
     PPosOverride = f;
   }
+  MyXUngrabServer(dpy);
 
   return;
 }
@@ -903,8 +944,8 @@ static void hide_screen(
     {
       /* When recapturing, all windows are reparented to this window. If they
        * are reparented to the root window, they will flash over the hide_win
-       * with XFree.  So reparent them to an unmapped window that looks like the
-       * root window. */
+       * with XFree.  So reparent them to an unmapped window that looks like
+       * the root window. */
       parent_win = XCreateWindow(
 	dpy, Scr.Root, 0, 0, Scr.MyDisplayWidth, Scr.MyDisplayHeight, 0,
 	CopyFromParent, InputOutput, CopyFromParent, valuemask, &xswa);
@@ -1018,7 +1059,6 @@ void CaptureAllWindows(void)
    * don't use PPosition */
   PPosOverride = False;
   MyXUngrabServer(dpy);
-  XSync(dpy,0); /* should we do this on initial capture? */
 }
 
 /*
@@ -1631,9 +1671,6 @@ static void InitVariables(void)
   Menus.all = NULL;
   Menus.DefaultStyle = NULL;
   Menus.LastStyle = NULL;
-  Menus.PopupDelay10ms = DEFAULT_POPUP_DELAY;
-  Menus.DoubleClickTime = DEFAULT_MENU_CLICKTIME;
-  memset(&Menus.flags, 0, sizeof(Menus.flags));
   Scr.last_added_item.type = ADDED_NONE;
 
   Scr.DefaultIcon = NULL;
@@ -1784,6 +1821,11 @@ static void Reborder(void)
   for (tmp = Scr.FvwmRoot.stack_prev; tmp != &Scr.FvwmRoot;
        tmp = tmp->stack_prev)
   {
+    if (!IS_ICONIFIED(tmp) && Scr.CurrentDesk != tmp->Desk)
+    {
+      XMapWindow(dpy, tmp->w);
+      SetMapStateProp(tmp, NormalState);
+    }
     RestoreWithdrawnLocation (tmp, True, Scr.Root);
     XUnmapWindow(dpy,tmp->frame);
     XDestroyWindow(dpy,tmp->frame);
@@ -1819,6 +1861,11 @@ void Done(int restart, char *command)
 {
   const char *exitFuncName;
 
+  /* XFree freeze hack */
+  XUngrabPointer(dpy, CurrentTime);
+  XUngrabKeyboard(dpy, CurrentTime);
+  XUngrabServer(dpy);
+
   if (!restart)
   {
     MoveViewport(0,0,False);
@@ -1870,7 +1917,9 @@ void Done(int restart, char *command)
 
     /* Really make sure that the connection is closed and cleared! */
     CloseICCCM2();
+    catch_exit();
     XCloseDisplay(dpy);
+    dpy = NULL;
 
     /* really need to destroy all windows, explicitly,
      * not sleep, but this is adequate for now */
@@ -1932,7 +1981,9 @@ void Done(int restart, char *command)
   else
   {
     CloseICCCM2();
+    catch_exit();
     XCloseDisplay(dpy);
+    dpy = NULL;
   }
 
   /* Close all my pipes */
