@@ -59,11 +59,12 @@
 #include "move_resize.h"
 #include "add_window.h"
 #include "module_interface.h"
+#include "focus.h"
 
 static void init_style(
   FvwmWindow *old_t, FvwmWindow *t, window_style *pstyle, short *pbuttons)
 {
-  /* copy window structure because we still need some old values */
+  /* copy the window structure because we still need some old values. */
   memcpy(old_t, t, sizeof(FvwmWindow));
   /* determine level of decoration */
   setup_style_and_decor(t, pstyle, pbuttons);
@@ -76,6 +77,11 @@ static void apply_window_updates(
   short buttons;
   Bool is_style_initialised = False;
 
+  if (flags->do_update_window_grabs)
+  {
+    focus_grab_buttons(t, False);
+  }
+
   if (IS_TRANSIENT(t) && flags->do_redecorate_transient)
   {
     flags->do_redecorate = True;
@@ -119,8 +125,7 @@ static void apply_window_updates(
       init_style(&old_t, t, pstyle, &buttons);
       is_style_initialised = True;
     }
-    setup_window_font(
-      t, pstyle, (flags->do_update_window_font && HAS_WINDOW_FONT(t)));
+    setup_window_font(t, pstyle, flags->do_update_window_font);
     flags->do_redecorate = True;
   }
   if (flags->do_redecorate ||
@@ -238,8 +243,7 @@ static void apply_window_updates(
       init_style(&old_t, t, pstyle, &buttons);
       is_style_initialised = True;
     }
-    setup_icon_font(
-      t, pstyle, (flags->do_update_icon_font && HAS_ICON_FONT(t)));
+    setup_icon_font(t, pstyle, flags->do_update_icon_font);
     flags->do_update_icon_title = True;
   }
   if (flags->do_update_icon_boxes)
@@ -277,6 +281,38 @@ static void apply_window_updates(
   return;
 }
 
+/* similar, but takes only care of destroying windows that have to go away. */
+void destroy_scheduled_windows(void)
+{
+  FvwmWindow *t;
+  FvwmWindow *next;
+  Bool do_need_ungrab = False;
+
+  if (Scr.flags.is_executing_complex_function != 0 ||
+      Scr.flags.is_window_scheduled_for_destroy == 0)
+    return;
+  /* Grab the server during the style update! */
+  if (GrabEm(CRS_WAIT, GRAB_BUSY))
+    do_need_ungrab = True;
+  MyXGrabServer(dpy);
+  Scr.flags.is_window_scheduled_for_destroy = 0;
+  /* need to destroy one or more windows before looking at the window list */
+  for (t = Scr.FvwmRoot.next, next = NULL; t != NULL; t = next)
+  {
+    next = t->next;
+    if (IS_SCHEDULED_FOR_DESTROY(t))
+    {
+      SET_SCHEDULED_FOR_DESTROY(t, 0);
+      destroy_window(t);
+    }
+  }
+  MyXUngrabServer(dpy);
+  if (do_need_ungrab)
+    UngrabEm(GRAB_BUSY);
+
+  return;
+}
+
 /* similar to the flush_window_updates() function, but does only the updates
  * for a single window whose decor has been changed. */
 void apply_decor_change(FvwmWindow *tmp_win)
@@ -296,7 +332,6 @@ void flush_window_updates(void)
 {
   FvwmWindow *t;
   window_style style;
-  Window focus_w;
   FvwmWindow *focus_fw;
   Bool do_need_ungrab = False;
   update_win flags;
@@ -306,13 +341,11 @@ void flush_window_updates(void)
   if (GrabEm(CRS_WAIT, GRAB_BUSY))
     do_need_ungrab = True;
   MyXGrabServer(dpy);
-  XSync(dpy,0);
 
   /* This is necessary in case the focus policy changes. With ClickToFocus some
    * buttons have to be grabbed/ungrabbed. */
   focus_fw = Scr.Focus;
-  focus_w = (focus_fw) ? focus_fw->w : Scr.NoFocusWin;
-  SetFocus(Scr.NoFocusWin, NULL, 1);
+  DeleteFocus(1);
 
   /* Apply the new default font and colours first */
   if (Scr.flags.has_default_color_changed || Scr.flags.has_default_font_changed)
@@ -324,7 +357,7 @@ void flush_window_updates(void)
     check_window_style_change(t, &flags, &style);
     if (Scr.flags.has_nr_buttons_changed)
       flags.do_redecorate = True;
-    /*!!!this is not optimised for minimal redrawing yet*/
+    /* TODO: this is not optimised for minimal redrawing yet*/
     if (t->decor->flags.has_changed)
     {
       flags.do_redecorate = True;
@@ -342,25 +375,39 @@ void flush_window_updates(void)
     {
       flags.do_update_window_font_height = True;
     }
+    if (Scr.flags.has_mouse_binding_changed)
+    {
+      flags.do_update_window_grabs = True;
+    }
     /* now apply the changes */
     apply_window_updates(t, &flags, &style, focus_fw);
   }
 
   /* restore the focus; also handles the case that the previously focused
    * window is now NeverFocus */
-  SetFocus(focus_w, focus_fw, 1);
+  if (focus_fw)
+  {
+    SetFocusWindow(focus_fw, 1);
+    if (Scr.flags.has_mouse_binding_changed)
+    {
+      focus_grab_buttons(focus_fw, True);
+    }
+  }
+  else
+    DeleteFocus(1);
+
   /* finally clean up the change flags */
   reset_style_changes();
   reset_decor_changes();
   Scr.flags.do_need_window_update = 0;
   Scr.flags.has_default_font_changed = 0;
   Scr.flags.has_default_color_changed = 0;
+  Scr.flags.has_mouse_binding_changed = 0;
   Scr.flags.has_nr_buttons_changed = 0;
 
   MyXUngrabServer(dpy);
   if (do_need_ungrab)
     UngrabEm(GRAB_BUSY);
-  XSync(dpy, 0);
 
   return;
 }
