@@ -92,16 +92,16 @@ char *WindowHiBack = NULL;
 char *WindowHiFore = NULL;
 char *WindowLabelFormat = NULL;
 
-unsigned int WindowBorderWidth = 1;
-unsigned int MinSize = 3;
+unsigned int WindowBorderWidth = DEFAULT_PAGER_WINDOW_BORDER_WIDTH;
+unsigned int MinSize = 2 * DEFAULT_PAGER_WINDOW_BORDER_WIDTH + 1;
 Bool WindowBorders3d = False;
 
 Picture *PixmapBack = NULL;
 
 char *ImagePath = NULL;
 
-#define DEFAULT_MOVE_THRESHOLD 3
-int MoveThreshold = DEFAULT_MOVE_THRESHOLD;
+#define DEFAULT_PAGER_MOVE_THRESHOLD 3
+int MoveThreshold = DEFAULT_PAGER_MOVE_THRESHOLD;
 
 int ShowBalloons = 0, ShowPagerBalloons = 0, ShowIconBalloons = 0;
 Bool do_focus_on_enter = False;
@@ -280,12 +280,11 @@ int main(int argc, char **argv)
   ndesks = desk2 - desk1 + 1;
 
   Desks = (DeskInfo *)safemalloc(ndesks*sizeof(DeskInfo));
+  memset(Desks, 0, ndesks * sizeof(DeskInfo));
   for(i=0;i<ndesks;i++)
     {
       sprintf(line,"Desk %d",i+desk1);
       CopyString(&Desks[i].label,line);
-      Desks[i].Dcolor = NULL;
-      Desks[i].bgPixmap = NULL;
       Desks[i].colorset = -1;
       Desks[i].highcolorset = -1;
       Desks[i].ballooncolorset = -1;
@@ -576,18 +575,7 @@ void list_add(unsigned long *body)
       i++;
     }
   *prev = (PagerWindow *)safemalloc(sizeof(PagerWindow));
-  (*prev)->next = NULL;
-  (*prev)->pager_view_width = 0;
-  (*prev)->pager_view_height = 0;
-  (*prev)->icon_view_width = 0;
-  (*prev)->icon_view_height = 0;
-  (*prev)->icon_name = NULL;
-  (*prev)->window_name = NULL;
-  (*prev)->res_name = NULL;
-  (*prev)->res_class = NULL;
-  (*prev)->window_label = NULL;
-  (*prev)->mini_icon.picture = 0;
-
+  memset(*prev, 0, sizeof(PagerWindow));
   (*prev)->w = cfgpacket->w;
   (*prev)->frame = cfgpacket->frame;
   (*prev)->t = (char *) cfgpacket->fvwmwin;
@@ -604,13 +592,13 @@ void list_add(unsigned long *body)
 
   if (win_pix_set)
   {
-        (*prev)->text = win_fore_pix;
-        (*prev)->back = win_back_pix;
+    (*prev)->text = win_fore_pix;
+    (*prev)->back = win_back_pix;
   }
   else
   {
-        (*prev)->text = body[22];
-        (*prev)->back = body[23];
+    (*prev)->text = body[22];
+    (*prev)->back = body[23];
   }
   AddNewWindow(*prev);
 }
@@ -630,14 +618,14 @@ void list_configure(unsigned long *body)
   target_w = cfgpacket->w;
   t = Start;
   while((t!= NULL)&&(t->w != target_w))
-    {
-      t = t->next;
-    }
+  {
+    t = t->next;
+  }
   if(t== NULL)
-    {
-      list_add(body);
-      return;
-    }
+  {
+    list_add(body);
+    return;
+  }
 
   t->t = (char *) cfgpacket->fvwmwin;
   t->frame = cfgpacket->frame;
@@ -717,7 +705,14 @@ void list_destroy(unsigned long *body)
       XDestroyWindow(dpy,t->IconView);
       if(FocusWin == t)
 	FocusWin = NULL;
-
+      if(t->res_class != NULL)
+	free(t->res_class);
+      if(t->res_name != NULL)
+	free(t->res_name);
+      if(t->window_name != NULL)
+	free(t->window_name);
+      if(t->icon_name != NULL)
+	free(t->icon_name);
       free(t);
     }
 }
@@ -1210,43 +1205,43 @@ void list_restack(unsigned long *body, unsigned long length)
   wins = (Window *) safemalloc (length * sizeof (Window));
   /* first restack in the icon view */
   j = 0;
-  for (i = 0; i < (length - 4); i+=3)
+  for (i = 0; i < (length - FvwmPacketHeaderSize); i += 3)
   {
     target_w = body[i];
     t = Start;
     while((t!= NULL)&&(t->w != target_w))
-     {
-       t = t->next;
-     }
+    {
+      t = t->next;
+    }
     if (t != NULL)
-     {
-       wins[j++] = t->IconView;
-     }
-   }
-   XRestackWindows(dpy, wins, j);
+    {
+      wins[j++] = t->IconView;
+    }
+  }
+  XRestackWindows(dpy, wins, j);
 
   /* now restack each desk separately, since they have separate roots */
   for (d = 0; d < ndesks; d++)
-   {
-     j = 0;
-     for (i = 0; i < (length - 4); i+=3)
+  {
+    j = 0;
+    for (i = 0; i < (length - 4); i+=3)
+    {
+      target_w = body[i];
+      t = Start;
+      while((t!= NULL)&&((t->w != target_w)||(t->desk != d+desk1)))
       {
-        target_w = body[i];
-        t = Start;
-        while((t!= NULL)&&((t->w != target_w)||(t->desk != d+desk1)))
-         {
-            t = t->next;
-         }
-        if (t != NULL)
-         {
-           if (t->PagerView != None)
-            {
-             wins[j++] = t->PagerView;
-            }
-         }
-       }
-      XRestackWindows(dpy, wins, j);
+	t = t->next;
+      }
+      if (t != NULL)
+      {
+	if (t->PagerView != None)
+	{
+	  wins[j++] = t->PagerView;
+	}
+      }
     }
+    XRestackWindows(dpy, wins, j);
+  }
   free (wins);
 }
 
@@ -1497,7 +1492,7 @@ void ParseOptions(void)
 	  if (val >= 0)
 	    MoveThreshold = val;
 	  else
-	    MoveThreshold = DEFAULT_MOVE_THRESHOLD;
+	    MoveThreshold = DEFAULT_PAGER_MOVE_THRESHOLD;
 	}
       }
       continue;
@@ -2063,15 +2058,12 @@ PagerStringList *NewPagerStringItem(PagerStringList *last, int desk)
   PagerStringList *newitem;
 
   newitem = (PagerStringList *)safemalloc(sizeof(PagerStringList));
+  memset(newitem, 0, sizeof(PagerStringList));
   last->next = newitem;
   newitem->colorset = -1;
   newitem->highcolorset = -1;
   newitem->ballooncolorset = -1;
   newitem->desk = desk;
-  newitem->next = NULL;
-  newitem->label = NULL;
-  newitem->Dcolor = NULL;
-  newitem->bgPixmap = NULL;
 
   return newitem;
 }
