@@ -27,12 +27,12 @@ extern Display *dpy;
 extern int screen;
 extern X11base *x11base;
 extern int grab_serve;
-extern struct XObj *tabxobj[100];
+extern struct XObj *tabxobj[1000];
 extern void LoadIcon(struct XObj *xobj);
 
 extern int nbobj;
 extern char **TabVVar;
-extern int TabIdObj[101];
+extern int TabIdObj[1001];
 extern char *ScriptName;
 extern char *ModuleName;
 extern TypeBuffSend BuffSend;
@@ -296,7 +296,7 @@ static char *FuncGetOutput(int *NbArg, long *TabArg)
    BufCom=(char*)calloc(sizeof(char),maxsize);
    size=fread(BufCom,1,maxsize,f);
    pclose(f);
-   strncpy(Command,cmndbuf,256);
+   strncpy(Command,cmndbuf,255);
    TimeCom=time(NULL);
   }
  }
@@ -657,7 +657,7 @@ static char *ReceivFromScript (int *NbArg,long *TabArg)
  XConvertSelection(dpy,ASend,AReceiv,propriete,x11base->win,CurrentTime);
  while ((!XCheckTypedEvent(dpy,SelectionNotify,&event))&&(NbEssai<10))
  {
-  usleep(1);
+   usleep(1);
   NbEssai++;
  }
  if (event.xselection.property!=None)
@@ -857,7 +857,8 @@ static void ChangeFont (int NbArg,long *TabArg)
  i++;
  arg[1]=CalcArg(TabArg,&i);
  IdItem= TabIdObj[atoi(arg[0])];
-
+ if (tabxobj[IdItem]->font)
+   free(tabxobj[IdItem]->font);
  tabxobj[IdItem]->font=strdup(arg[1]);
 #ifdef I18N_MB
  /* Hmm.. Fontset is not freed. However, original alogrithm does not consider
@@ -928,6 +929,8 @@ static void ChangeTitle (int NbArg,long *TabArg)
  arg[1]=CalcArg(TabArg,&i);
  IdItem= TabIdObj[atoi(arg[0])];
 
+ if (tabxobj[IdItem]->title)
+   free(tabxobj[IdItem]->title);
  tabxobj[IdItem]->title=strdup(arg[1]);
  if (tabxobj[IdItem]->TypeWidget != SwallowExec)
    XClearWindow(dpy, tabxobj[IdItem]->win);
@@ -1066,6 +1069,8 @@ static void ChangeColorset (int NbArg,long *TabArg)
  arg[1]=CalcArg(TabArg,&i);
  if (atoi(arg[0]) == 0) {
    ChangeMainColorset(atoi(arg[1]));
+   free(arg[0]);
+   free(arg[1]);
    return;
  }
  IdItem= TabIdObj[atoi(arg[0])];
